# Copyright 2021 Katteli Inc.
# TestFlows.com Open-Source Software Testing Framework (http://testflows.com)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
def transform(msgs, stop=None, key="test_id"):
    """Sort messages by key.
    """
    def sorting_key(msg):
        return msg[key]

    msgs.sort(key=sorting_key)

    length = len(msgs)

    yield None

    for i, msg in enumerate(msgs):
        if stop:
            if i + 1 == length:
                stop.set()
        yield msg
